# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TrackEventParams", "DefaultProperties", "UserProperties"]


class TrackEventParams(TypedDict, total=False):
    token: Required[str]
    """The token for your Ours Privacy Source.

    You can find this in the Ours dashboard.
    """

    event: Required[str]
    """The name of the event you're tracking.

    This must be whitelisted in the Ours dashboard.
    """

    default_properties: Annotated[Optional[DefaultProperties], PropertyInfo(alias="defaultProperties")]
    """
    These properties are used throughout the Ours app to pass known values onto
    destinations
    """

    distinct_id: Annotated[Optional[str], PropertyInfo(alias="distinctId")]
    """A unique identifier for the event. This helps prevent duplicate events."""

    email: Optional[str]
    """The email address of a user.

    We will associate this event with the user or create a user. Used for lookup if
    externalId and userId are not included in the request.
    """

    event_properties: Annotated[Optional[Dict[str, Optional[object]]], PropertyInfo(alias="eventProperties")]
    """Any additional event properties you want to pass along."""

    external_id: Annotated[Optional[str], PropertyInfo(alias="externalId")]
    """The externalId (the ID in your system) of a user.

    We will associate this event with the user or create a user. If included in the
    request, email lookup is ignored.
    """

    time: Optional[float]
    """The time at which the event occurred in milliseconds since UTC epoch.

    The time must be in the past and within the last 7 days.
    """

    user_id: Annotated[Optional[str], PropertyInfo(alias="userId")]
    """The Ours user id stored in local storage and cookies on your web properties.

    If userId is included in the request, we do not lookup the user by email or
    externalId.
    """

    user_properties: Annotated[Optional[UserProperties], PropertyInfo(alias="userProperties")]
    """Properties to set on the visitor.

    (optional) You can also update these properties via the identify endpoint.
    """


class DefaultProperties(TypedDict, total=False):
    active_duration: Annotated[Optional[float], PropertyInfo(alias="activeDuration")]
    """The active time in milliseconds that the user had this tab active"""

    ad_id: Optional[str]
    """The ad id for detected in the session.

    This is set by the web sdk automatically.
    """

    adset_id: Optional[str]
    """The adset id for detected in the session.

    This is set by the web sdk automatically.
    """

    browser_language: Optional[str]
    """The language of the browser. Ex: en-US"""

    browser_name: Optional[str]
    """The name of the browser. Ex: Chrome"""

    browser_version: Optional[str]
    """The version of the browser. Ex: 114.0"""

    campaign_id: Optional[str]
    """The campaign id for detected in the session.

    This is set by the web sdk automatically.
    """

    clickid: Optional[str]
    """The Click ID. Ex: clickid123"""

    clid: Optional[str]
    """The Generic Click ID. Ex: clid123"""

    cpu_architecture: Optional[str]
    """The architecture of the CPU. Ex: x64"""

    current_url: Optional[str]
    """The full url (including query params) of the current page"""

    dclid: Optional[str]
    """The DoubleClick Click ID. Ex: dclid123"""

    device_model: Optional[str]
    """The model of the device. Ex: iPhone 13"""

    device_type: Optional[str]
    """The type of device the user is using. Ex: mobile"""

    device_vendor: Optional[str]
    """The vendor of the device. Ex: Apple"""

    duration: Optional[float]
    """The time in milliseconds since the page was loaded // script was loaded"""

    encoding: Optional[str]
    """The browsers encoding. Ex: UTF-8"""

    engine_name: Optional[str]
    """The name of the browser engine. Ex: Blink"""

    engine_version: Optional[str]
    """The version of the browser engine. Ex: 114.0"""

    epik: Optional[str]
    """The Pinterest Click ID. Ex: epik456"""

    fbc: Optional[str]
    """Facebook Click ID with prefix format for Conversions API tracking.

    Ex: fb.1.1554763741205.AbCdEfGhIjKlMnOpQrStUvWxYz1234567890
    """

    fbclid: Optional[str]
    """Raw Facebook Click ID query parameter without prefix from ad clicks.

    Ex: AbCdEfGhIjKlMnOpQrStUvWxYz1234567890
    """

    fbp: Optional[str]
    """Facebook Browser ID parameter for identifying browsers and attributing events.

    Ex: fb.1.1554763741205.1098115397
    """

    fv: Optional[bool]
    """Deprecated"""

    gad_source: Optional[str]
    """The Google Ad Source. Ex: google"""

    gbraid: Optional[str]
    """The Google Braid ID. Ex: gbraid123"""

    gclid: Optional[str]
    """The Google Click ID. Ex: gclid123"""

    host: Optional[str]
    """The host of the current page. Ex: example.com"""

    iframe: Optional[bool]
    """Whether the user is in an iframe. Ex: true"""

    ip: Optional[str]
    """The IP address of the user. Ex: 127.0.0.1"""

    is_bot: object
    """Whether we have detected that the user is a bot.

    This is set automatically by the Ours server primarily for events tracked
    through the web SDK.
    """

    li_fat_id: Optional[str]
    """The LinkedIn Click ID. Ex: li_fat_id123"""

    msclkid: Optional[str]
    """The Microsoft Click ID. Ex: msclkid123"""

    ndclid: Optional[str]
    """The NextDoor Click ID. Ex: ndclid123"""

    new_s: Optional[bool]
    """Deprecated"""

    os_name: Optional[str]
    """The name of the operating system. Ex: Windows"""

    os_version: Optional[str]
    """The version of the operating system. Ex: 10.0"""

    page_hash: Optional[float]
    """A random set of numbers for the page load"""

    pathname: Optional[str]
    """The pathname of the current page. Ex: /home"""

    qclid: Optional[str]
    """The Quora Click ID. Ex: qclid123"""

    rdt_cid: Optional[str]
    """The Reddit Click ID. Ex: rdt_cid123"""

    received_at: Optional[str]
    """The time the event was received by an Ours server in ISO format"""

    referrer: Optional[str]
    """The referrer URL of the current page"""

    sacid: Optional[str]
    """The StackAdapt Tracking ID. Ex: sacid123"""

    sccid: Optional[str]
    """The SnapChat Click ID. Ex: sccid123"""

    screen_height: Optional[float]
    """The height of the screen. Ex: 1080"""

    screen_width: Optional[float]
    """The width of the screen. Ex: 1920"""

    session_count: Annotated[Optional[float], PropertyInfo(alias="sessionCount")]
    """The number of sessions the user has had. Ex: 3"""

    sid: Optional[str]
    """The session ID as assigned automatically by the web SDK.

    This is required for session replay
    """

    sr: Optional[str]

    title: Optional[str]
    """The title of the current page"""

    ttclid: Optional[str]
    """The TikTok Click ID. Ex: ttclid123"""

    twclid: Optional[str]
    """The Twitter Click ID. Ex: twclid123"""

    uafvl: Optional[str]
    """User agent as a full list of strings."""

    user_agent: Optional[str]
    """The user agent of the browser"""

    utm_campaign: Optional[str]
    """The UTM Campaign.

    The web SDK automatically captures this from the query params.
    """

    utm_content: Optional[str]
    """The UTM Content. The web SDK automatically captures this from the query params."""

    utm_medium: Optional[str]
    """The UTM Medium. The web SDK automatically captures this from the query params."""

    utm_name: Optional[str]
    """The UTM Name. The web SDK automatically captures this from the query params."""

    utm_source: Optional[str]
    """The UTM Source. The web SDK automatically captures this from the query params."""

    utm_term: Optional[str]
    """The UTM Term. The web SDK automatically captures this from the query params."""

    version: Optional[str]
    """The version of the web SDK"""

    wbraid: Optional[str]
    """The WBRAID Identifier.

    The web SDK automatically captures this from the query params.
    """

    webview: Optional[bool]
    """Whether the user is in a webview. Ex: true"""


class UserProperties(TypedDict, total=False):
    ad_id: Optional[str]

    adset_id: Optional[str]

    campaign_id: Optional[str]

    city: Optional[str]

    clickid: Optional[str]

    clid: Optional[str]

    company_name: Optional[str]

    consent: Optional[Dict[str, Optional[object]]]

    country: Optional[str]

    custom_properties: Optional[Dict[str, Optional[object]]]

    date_of_birth: Optional[str]

    dclid: Optional[str]

    email: Optional[str]

    epik: Optional[str]

    external_id: Optional[str]

    fbc: Optional[str]

    fbclid: Optional[str]

    fbp: Optional[str]

    first_name: Optional[str]

    gad_source: Optional[str]

    gbraid: Optional[str]

    gclid: Optional[str]

    gender: Optional[str]

    ip: Optional[str]
    """The IP address of the user"""

    is_bot: object

    job_title: Optional[str]

    last_name: Optional[str]

    li_fat_id: Optional[str]

    msclkid: Optional[str]

    ndclid: Optional[str]

    phone_number: object

    qclid: Optional[str]

    rdt_cid: Optional[str]

    referrer: Optional[str]

    sacid: Optional[str]

    sccid: Optional[str]

    sid: Optional[str]

    state: Optional[str]

    ttclid: Optional[str]

    twclid: Optional[str]

    user_agent: Optional[str]

    user_agent_full_list: Optional[str]

    utm_campaign: Optional[str]

    utm_content: Optional[str]

    utm_medium: Optional[str]

    utm_name: Optional[str]

    utm_source: Optional[str]

    utm_term: Optional[str]

    wbraid: Optional[str]

    zip: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from ours_privacy import OursPrivacy, AsyncOursPrivacy
from ours_privacy.types import TrackEventResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTrack:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_event(self, client: OursPrivacy) -> None:
        track = client.track.event(
            token="x",
            event="x",
        )
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_event_with_all_params(self, client: OursPrivacy) -> None:
        track = client.track.event(
            token="x",
            event="x",
            default_properties={
                "active_duration": 0,
                "ad_id": "ad_id",
                "adset_id": "adset_id",
                "browser_language": "browser_language",
                "browser_name": "browser_name",
                "browser_version": "browser_version",
                "campaign_id": "campaign_id",
                "clickid": "clickid",
                "clid": "clid",
                "cpu_architecture": "cpu_architecture",
                "current_url": "current_url",
                "dclid": "dclid",
                "device_model": "device_model",
                "device_type": "device_type",
                "device_vendor": "device_vendor",
                "duration": 0,
                "encoding": "encoding",
                "engine_name": "engine_name",
                "engine_version": "engine_version",
                "epik": "epik",
                "fbc": "fbc",
                "fbclid": "fbclid",
                "fbp": "fbp",
                "fv": True,
                "gad_source": "gad_source",
                "gbraid": "gbraid",
                "gclid": "gclid",
                "host": "host",
                "iframe": True,
                "ip": "ip",
                "is_bot": {},
                "li_fat_id": "li_fat_id",
                "msclkid": "msclkid",
                "ndclid": "ndclid",
                "new_s": True,
                "os_name": "os_name",
                "os_version": "os_version",
                "page_hash": 0,
                "pathname": "pathname",
                "qclid": "qclid",
                "rdt_cid": "rdt_cid",
                "received_at": "received_at",
                "referrer": "referrer",
                "sacid": "sacid",
                "sccid": "sccid",
                "screen_height": 0,
                "screen_width": 0,
                "session_count": 0,
                "sid": "sid",
                "sr": "sr",
                "title": "title",
                "ttclid": "ttclid",
                "twclid": "twclid",
                "uafvl": "uafvl",
                "user_agent": "user_agent",
                "utm_campaign": "utm_campaign",
                "utm_content": "utm_content",
                "utm_medium": "utm_medium",
                "utm_name": "utm_name",
                "utm_source": "utm_source",
                "utm_term": "utm_term",
                "version": "version",
                "wbraid": "wbraid",
                "webview": True,
            },
            distinct_id="x",
            email="x",
            event_properties={"foo": "bar"},
            external_id="x",
            time=0,
            user_id="x",
            user_properties={
                "ad_id": "ad_id",
                "adset_id": "adset_id",
                "campaign_id": "campaign_id",
                "city": "city",
                "clickid": "clickid",
                "clid": "clid",
                "company_name": "company_name",
                "consent": {"foo": "bar"},
                "country": "country",
                "custom_properties": {"foo": "bar"},
                "date_of_birth": "date_of_birth",
                "dclid": "dclid",
                "email": "email",
                "epik": "epik",
                "external_id": "external_id",
                "fbc": "fbc",
                "fbclid": "fbclid",
                "fbp": "fbp",
                "first_name": "first_name",
                "gad_source": "gad_source",
                "gbraid": "gbraid",
                "gclid": "gclid",
                "gender": "gender",
                "ip": "ip",
                "is_bot": {},
                "job_title": "job_title",
                "last_name": "last_name",
                "li_fat_id": "li_fat_id",
                "msclkid": "msclkid",
                "ndclid": "ndclid",
                "phone_number": {},
                "qclid": "qclid",
                "rdt_cid": "rdt_cid",
                "referrer": "referrer",
                "sacid": "sacid",
                "sccid": "sccid",
                "sid": "sid",
                "state": "state",
                "ttclid": "ttclid",
                "twclid": "twclid",
                "user_agent": "user_agent",
                "user_agent_full_list": "user_agent_full_list",
                "utm_campaign": "utm_campaign",
                "utm_content": "utm_content",
                "utm_medium": "utm_medium",
                "utm_name": "utm_name",
                "utm_source": "utm_source",
                "utm_term": "utm_term",
                "wbraid": "wbraid",
                "zip": {},
            },
        )
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_event(self, client: OursPrivacy) -> None:
        response = client.track.with_raw_response.event(
            token="x",
            event="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = response.parse()
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_event(self, client: OursPrivacy) -> None:
        with client.track.with_streaming_response.event(
            token="x",
            event="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            track = response.parse()
            assert_matches_type(TrackEventResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTrack:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_event(self, async_client: AsyncOursPrivacy) -> None:
        track = await async_client.track.event(
            token="x",
            event="x",
        )
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_event_with_all_params(self, async_client: AsyncOursPrivacy) -> None:
        track = await async_client.track.event(
            token="x",
            event="x",
            default_properties={
                "active_duration": 0,
                "ad_id": "ad_id",
                "adset_id": "adset_id",
                "browser_language": "browser_language",
                "browser_name": "browser_name",
                "browser_version": "browser_version",
                "campaign_id": "campaign_id",
                "clickid": "clickid",
                "clid": "clid",
                "cpu_architecture": "cpu_architecture",
                "current_url": "current_url",
                "dclid": "dclid",
                "device_model": "device_model",
                "device_type": "device_type",
                "device_vendor": "device_vendor",
                "duration": 0,
                "encoding": "encoding",
                "engine_name": "engine_name",
                "engine_version": "engine_version",
                "epik": "epik",
                "fbc": "fbc",
                "fbclid": "fbclid",
                "fbp": "fbp",
                "fv": True,
                "gad_source": "gad_source",
                "gbraid": "gbraid",
                "gclid": "gclid",
                "host": "host",
                "iframe": True,
                "ip": "ip",
                "is_bot": {},
                "li_fat_id": "li_fat_id",
                "msclkid": "msclkid",
                "ndclid": "ndclid",
                "new_s": True,
                "os_name": "os_name",
                "os_version": "os_version",
                "page_hash": 0,
                "pathname": "pathname",
                "qclid": "qclid",
                "rdt_cid": "rdt_cid",
                "received_at": "received_at",
                "referrer": "referrer",
                "sacid": "sacid",
                "sccid": "sccid",
                "screen_height": 0,
                "screen_width": 0,
                "session_count": 0,
                "sid": "sid",
                "sr": "sr",
                "title": "title",
                "ttclid": "ttclid",
                "twclid": "twclid",
                "uafvl": "uafvl",
                "user_agent": "user_agent",
                "utm_campaign": "utm_campaign",
                "utm_content": "utm_content",
                "utm_medium": "utm_medium",
                "utm_name": "utm_name",
                "utm_source": "utm_source",
                "utm_term": "utm_term",
                "version": "version",
                "wbraid": "wbraid",
                "webview": True,
            },
            distinct_id="x",
            email="x",
            event_properties={"foo": "bar"},
            external_id="x",
            time=0,
            user_id="x",
            user_properties={
                "ad_id": "ad_id",
                "adset_id": "adset_id",
                "campaign_id": "campaign_id",
                "city": "city",
                "clickid": "clickid",
                "clid": "clid",
                "company_name": "company_name",
                "consent": {"foo": "bar"},
                "country": "country",
                "custom_properties": {"foo": "bar"},
                "date_of_birth": "date_of_birth",
                "dclid": "dclid",
                "email": "email",
                "epik": "epik",
                "external_id": "external_id",
                "fbc": "fbc",
                "fbclid": "fbclid",
                "fbp": "fbp",
                "first_name": "first_name",
                "gad_source": "gad_source",
                "gbraid": "gbraid",
                "gclid": "gclid",
                "gender": "gender",
                "ip": "ip",
                "is_bot": {},
                "job_title": "job_title",
                "last_name": "last_name",
                "li_fat_id": "li_fat_id",
                "msclkid": "msclkid",
                "ndclid": "ndclid",
                "phone_number": {},
                "qclid": "qclid",
                "rdt_cid": "rdt_cid",
                "referrer": "referrer",
                "sacid": "sacid",
                "sccid": "sccid",
                "sid": "sid",
                "state": "state",
                "ttclid": "ttclid",
                "twclid": "twclid",
                "user_agent": "user_agent",
                "user_agent_full_list": "user_agent_full_list",
                "utm_campaign": "utm_campaign",
                "utm_content": "utm_content",
                "utm_medium": "utm_medium",
                "utm_name": "utm_name",
                "utm_source": "utm_source",
                "utm_term": "utm_term",
                "wbraid": "wbraid",
                "zip": {},
            },
        )
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_event(self, async_client: AsyncOursPrivacy) -> None:
        response = await async_client.track.with_raw_response.event(
            token="x",
            event="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = await response.parse()
        assert_matches_type(TrackEventResponse, track, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_event(self, async_client: AsyncOursPrivacy) -> None:
        async with async_client.track.with_streaming_response.event(
            token="x",
            event="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            track = await response.parse()
            assert_matches_type(TrackEventResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..types import track_event_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.track_event_response import TrackEventResponse

__all__ = ["TrackResource", "AsyncTrackResource"]


class TrackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TrackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/with-ours/ingest-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TrackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/with-ours/ingest-sdk-python#with_streaming_response
        """
        return TrackResourceWithStreamingResponse(self)

    def event(
        self,
        *,
        token: str,
        event: str,
        default_properties: Optional[track_event_params.DefaultProperties] | Omit = omit,
        distinct_id: Optional[str] | Omit = omit,
        email: Optional[str] | Omit = omit,
        event_properties: Optional[Dict[str, Optional[object]]] | Omit = omit,
        external_id: Optional[str] | Omit = omit,
        time: Optional[float] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        user_properties: Optional[track_event_params.UserProperties] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackEventResponse:
        """Track events from your server.

        Please include at least one of: userId,
        externalId, or email. These properties help us associate events with existing
        users. For all fields, null values unset the property and undefined values do
        not unset existing properties.

        Args:
          token: The token for your Ours Privacy Source. You can find this in the Ours dashboard.

          event: The name of the event you're tracking. This must be whitelisted in the Ours
              dashboard.

          default_properties: These properties are used throughout the Ours app to pass known values onto
              destinations

          distinct_id: A unique identifier for the event. This helps prevent duplicate events.

          email: The email address of a user. We will associate this event with the user or
              create a user. Used for lookup if externalId and userId are not included in the
              request.

          event_properties: Any additional event properties you want to pass along.

          external_id: The externalId (the ID in your system) of a user. We will associate this event
              with the user or create a user. If included in the request, email lookup is
              ignored.

          time: The time at which the event occurred in milliseconds since UTC epoch. The time
              must be in the past and within the last 7 days.

          user_id: The Ours user id stored in local storage and cookies on your web properties. If
              userId is included in the request, we do not lookup the user by email or
              externalId.

          user_properties: Properties to set on the visitor. (optional) You can also update these
              properties via the identify endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/track" if self._client._base_url_overridden else "https://api.oursprivacy.com/api/v1/track",
            body=maybe_transform(
                {
                    "token": token,
                    "event": event,
                    "default_properties": default_properties,
                    "distinct_id": distinct_id,
                    "email": email,
                    "event_properties": event_properties,
                    "external_id": external_id,
                    "time": time,
                    "user_id": user_id,
                    "user_properties": user_properties,
                },
                track_event_params.TrackEventParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackEventResponse,
        )


class AsyncTrackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTrackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/with-ours/ingest-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/with-ours/ingest-sdk-python#with_streaming_response
        """
        return AsyncTrackResourceWithStreamingResponse(self)

    async def event(
        self,
        *,
        token: str,
        event: str,
        default_properties: Optional[track_event_params.DefaultProperties] | Omit = omit,
        distinct_id: Optional[str] | Omit = omit,
        email: Optional[str] | Omit = omit,
        event_properties: Optional[Dict[str, Optional[object]]] | Omit = omit,
        external_id: Optional[str] | Omit = omit,
        time: Optional[float] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        user_properties: Optional[track_event_params.UserProperties] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackEventResponse:
        """Track events from your server.

        Please include at least one of: userId,
        externalId, or email. These properties help us associate events with existing
        users. For all fields, null values unset the property and undefined values do
        not unset existing properties.

        Args:
          token: The token for your Ours Privacy Source. You can find this in the Ours dashboard.

          event: The name of the event you're tracking. This must be whitelisted in the Ours
              dashboard.

          default_properties: These properties are used throughout the Ours app to pass known values onto
              destinations

          distinct_id: A unique identifier for the event. This helps prevent duplicate events.

          email: The email address of a user. We will associate this event with the user or
              create a user. Used for lookup if externalId and userId are not included in the
              request.

          event_properties: Any additional event properties you want to pass along.

          external_id: The externalId (the ID in your system) of a user. We will associate this event
              with the user or create a user. If included in the request, email lookup is
              ignored.

          time: The time at which the event occurred in milliseconds since UTC epoch. The time
              must be in the past and within the last 7 days.

          user_id: The Ours user id stored in local storage and cookies on your web properties. If
              userId is included in the request, we do not lookup the user by email or
              externalId.

          user_properties: Properties to set on the visitor. (optional) You can also update these
              properties via the identify endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/track" if self._client._base_url_overridden else "https://api.oursprivacy.com/api/v1/track",
            body=await async_maybe_transform(
                {
                    "token": token,
                    "event": event,
                    "default_properties": default_properties,
                    "distinct_id": distinct_id,
                    "email": email,
                    "event_properties": event_properties,
                    "external_id": external_id,
                    "time": time,
                    "user_id": user_id,
                    "user_properties": user_properties,
                },
                track_event_params.TrackEventParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackEventResponse,
        )


class TrackResourceWithRawResponse:
    def __init__(self, track: TrackResource) -> None:
        self._track = track

        self.event = to_raw_response_wrapper(
            track.event,
        )


class AsyncTrackResourceWithRawResponse:
    def __init__(self, track: AsyncTrackResource) -> None:
        self._track = track

        self.event = async_to_raw_response_wrapper(
            track.event,
        )


class TrackResourceWithStreamingResponse:
    def __init__(self, track: TrackResource) -> None:
        self._track = track

        self.event = to_streamed_response_wrapper(
            track.event,
        )


class AsyncTrackResourceWithStreamingResponse:
    def __init__(self, track: AsyncTrackResource) -> None:
        self._track = track

        self.event = async_to_streamed_response_wrapper(
            track.event,
        )
